from django.apps import apps
from django.contrib import admin
from django.db import models
from django.shortcuts import render
from django.urls import reverse

from django_daisy.module_settings import DAISY_SETTINGS

# Remove default form fields for specific date and time fields
admin.options.FORMFIELD_FOR_DBFIELD_DEFAULTS.pop(models.DateTimeField, None)
admin.options.FORMFIELD_FOR_DBFIELD_DEFAULTS.pop(models.DateField, None)
admin.options.FORMFIELD_FOR_DBFIELD_DEFAULTS.pop(models.TimeField, None)


class DaisyAdminSite(admin.AdminSite):
    password_change_template = 'admin/registration/password_change_form.html'
    password_change_done_template = 'admin/registration/password_change_done.html'
    site_title = DAISY_SETTINGS.get('SITE_TITLE', 'django admin')
    site_header = DAISY_SETTINGS.get('SITE_HEADER', 'Administration')
    index_title = DAISY_SETTINGS.get('SITE_HEADER', 'hi, welcome to your dashboard')
    logo = DAISY_SETTINGS.get('SITE_LOGO', '/static/admin/img/daisyui-logomark.svg')

    def get_log_entries(self, request):
        from django.contrib.admin.models import LogEntry

        return LogEntry.objects.select_related("content_type", "user")

    def index(self, request, extra_context=None):
        """
        Display the main admin index page, which lists all of the installed
        apps that have been registered in this site.
        """

        # try:
        logentry_changelist_url = reverse('admin:admin_logentry_changelist')
        # except Exception:
        #     logentry_changelist_url = ''

        app_list = self.get_app_list(request)

        context = {
            **self.each_context(request),
            'latest_history': self.get_log_entries(request)[:15],
            "title": self.index_title,
            "app_list": app_list,
            'logentry_changelist_url': logentry_changelist_url,
            **(extra_context or {}),
        }

        request.current_app = self.name

        return render(
            request, [
                "admin/index.html"
            ], context
        )

    # Overriding the get_app_list method to sort apps and exclude hidden ones
    def get_app_list(self, request, app_label=None):
        """
        Return a sorted list of all installed apps that are registered in this site,
        excluding hidden apps and adding optional icons and dividers.
        """
        app_dict = self._build_app_dict(request, app_label)
        sorted_app_list = sorted(app_dict.values(), key=lambda x: x.get('priority', 0), reverse=True)
        return [app for app in sorted_app_list if not app.get('hide', False)]

    def _build_app_dict(self, request, label=None):
        """
        Builds and modifies the app dictionary to include icons and app grouping logic.
        """
        app_dict = super()._build_app_dict(request, label)

        if label:
            return app_dict

        modified_app_dict = app_dict.copy()
        override_apps_config = DAISY_SETTINGS.get('APPS_REORDER', {})

        for app_label, app_info in app_dict.items():
            # Add icon and divider title to each app
            app_info['icon'] = getattr(apps.get_app_config(app_label), 'icon', '')
            app_info['divider_title'] = getattr(apps.get_app_config(app_label), 'divider_title', '')

            if app_label in override_apps_config:
                app_info.update(override_apps_config[app_label])

        return modified_app_dict

    def each_context(self, request):
        context = super().each_context(request)

        return {
            **context,
            **DAISY_SETTINGS,
            'logo': self.get_logo(request)
        }

    def get_logo(self, request):
        return self.logo



import json

from django import forms
from django.db import models


class JsonEditorWidget(forms.Textarea):
    template_name = 'fields/json_editor_field.html'
    schema_template = {}

    def get_context(self, name, value, attrs):
        context = super().get_context(name, value, attrs)
        context['schema_template'] = self.schema_template
        return context

    def __init__(self, attrs=None, schema_template=None):
        if schema_template:
            self.schema_template = schema_template

        super().__init__(attrs)

    class Media:
        js = [
            'admin/fields/json-editor/jsoneditor.min.js',
            'admin/fields/json-editor/init.js'
        ]


class JsonEditorField(models.JSONField):
    schema = {}

    def __init__(self, *args, schema: dict, **kwargs):
        self.schema = schema
        super().__init__(*args, **kwargs)

    def formfield(self, **kwargs):
        kwargs.update({
            'widget': JsonEditorWidget(schema_template=self.schema),
        })
        return super(JsonEditorField, self).formfield(**kwargs)

    def deconstruct(self):
        name, path, args, kwargs = super().deconstruct()
        kwargs['schema'] = self.schema

        return name, path, args, kwargs



from django.db.models import CharField
from django.forms import fields


class CommaSepWidget(fields.TextInput):
    """
        tokenfield tags field widget
    """
    template_name = 'django/forms/widgets/comma_seperate.html'

    class Media:
        js = (
            'admin/panel/global_assets/js/plugins/forms/tags/tokenfield.min.js',
        )

    def __init__(self, attrs=None):
        default_attrs = {
            'class': 'tokenfield',
        }
        if attrs:
            default_attrs.update(attrs)
        super().__init__(default_attrs)


class CommaSepModelField(CharField):
    description = "keyword field with comma separate in CharField"

    def __init__(self, *args, db_collation=None, **kwargs):
        kwargs['max_length'] = 255
        super().__init__(*args, **kwargs)

    def formfield(self, **kwargs):
        # override widget field
        defaults = {'widget': CommaSepWidget}

        return super().formfield(**defaults)



from django.db.models import PositiveBigIntegerField
from django.forms import fields


class ThousandSepWidget(fields.TextInput):
    template_name = 'django/forms/widgets/thousand_sep.html'

    def format_value(self, value):
        # Format the value with thousand separators
        # e.g., 200000 => '200,000'
        return '{:,}'.format(int(value)) if value else None

    def value_from_datadict(self, data, files, name):
        if value := super().value_from_datadict(data, files, name):
            return value.replace(',', '')
        return ''

    class Media:
        js = (
            'admin/fields/thousand_sep_field.js',
        )


class ThousandSepField(PositiveBigIntegerField):
    description = "number field with thousand separate eg: 200,000"

    def formfield(self, **kwargs):
        return super().formfield(**{
            'widget': ThousandSepWidget,
        })

    # def from_db_value(self, value, expression, connection):
    #     if value is None:
    #         return value
    #     return locale.format_string("%d", value, grouping=True)

    def get_prep_value(self, value):
        # Convert formatted value back to integer for storage
        if isinstance(value, str):
            value = value.replace(',', '')
        return super().get_prep_value(value)

    def value_to_string(self, obj):
        value = self.value_from_object(obj)
        return self.to_python(value)



from importlib.metadata import version, PackageNotFoundError

from django.contrib.admin.decorators import action, display, register
from django.contrib.admin.filters import (
    AllValuesFieldListFilter,
    BooleanFieldListFilter,
    ChoicesFieldListFilter,
    DateFieldListFilter,
    EmptyFieldListFilter,
    FieldListFilter,
    ListFilter,
    RelatedFieldListFilter,
    RelatedOnlyFieldListFilter,
    SimpleListFilter,
)
from django.contrib.admin.options import (
    HORIZONTAL,
    VERTICAL,
    ModelAdmin,
    StackedInline,
    TabularInline,
)
from django.contrib.admin.sites import AdminSite
from django.contrib.admin.sites import site as admin_site
from django.utils.functional import LazyObject
from django.utils.module_loading import autodiscover_modules, import_string

support_show_facet = False
try:
    from django.contrib.admin.options import ShowFacets

    support_show_facet = True
except Exception:
    pass


class DefaultAdminSite(LazyObject):
    def _setup(self):
        AdminSiteClass = import_string('django_daisy.admin.DaisyAdminSite')
        self._wrapped = AdminSiteClass()

    def __repr__(self):
        return repr(self._wrapped)


site = DefaultAdminSite()

__all__ = [
    "action",
    "display",
    "register",
    "ModelAdmin",
    "HORIZONTAL",
    "VERTICAL",
    "StackedInline",
    "TabularInline",
    "AdminSite",
    "ListFilter",
    "SimpleListFilter",
    "FieldListFilter",
    "BooleanFieldListFilter",
    "RelatedFieldListFilter",
    "ChoicesFieldListFilter",
    "DateFieldListFilter",
    "AllValuesFieldListFilter",
    "EmptyFieldListFilter",
    "RelatedOnlyFieldListFilter",
    "autodiscover",
]

if support_show_facet:
    __all__ += ['ShowFacets']


def autodiscover():
    # autodiscover_modules("django_daisy", register_to=site)
    autodiscover_modules("admin", register_to=admin_site)


try:
    __version__ = version("django_daisy")
except PackageNotFoundError:
    # package is not installed
    pass



from django.db import models
from django import forms


class SummernoteField(models.TextField):

    def __init__(self, *args, column_type='mediumtext', **kwargs):
        # specifies column type choices are text | mediumtext | longtext
        # default is mediumtext
        self.column_type = column_type
        super(SummernoteField, self).__init__(*args, **kwargs)

    def formfield(self, **kwargs):
        kwargs['widget'] = forms.Textarea(attrs={'class': 'summernote'})
        return super().formfield(**{
            'max_length': self.max_length,
            **({} if self.choices is not None else {'widget': forms.Textarea}),
            **kwargs,
        })

    def db_type(self, connection):
        if connection.settings_dict['ENGINE'] == 'django.db.backends.mysql':
            return self.column_type
        elif connection.settings_dict['ENGINE'] == 'django.db.backends.postgresql':
            return 'text'

        return super(SummernoteField, self).db_type(connection)



from setuptools import setup, find_packages

from setuptools_scm import get_version

setup(
    name='django-daisy',
    version=get_version(),
    packages=find_packages(),
    use_scm_version=True,
    setup_requires=['setuptools-scm'],
    include_package_data=True,
    install_requires=[],
    description='A modern django dashboard built with daisyui',
    long_description=open('README.md').read(),
    long_description_content_type='text/markdown',
    url='https://github.com/hypy13/django-daisy',
    author='Hossein Yaghoubi',
    author_email='hossein.yaghoubi13@gmail.com',
    license='Apache 2.0',
    classifiers=[
        'Programming Language :: Python :: 3',
        'License :: OSI Approved :: Apache Software License',
        'Operating System :: OS Independent',
    ],
)



import re
from urllib.parse import unquote

from django import template
from django.contrib.admin.templatetags import admin_list
from django.contrib.auth import get_user_model
from django.urls import reverse
from django.utils.html import format_html
from django.utils.safestring import mark_safe

from django_daisy.module_settings import DAISY_SETTINGS


def custom_boolean_icon(field_val):
    # Define icon and badge class based on field value
    icon_class = {
        True: "text-base fa fa-circle-check text-success",
        False: "text-base fa fa-circle-xmark text-error",
        None: "text-base fa fa-question-circle",
    }[field_val]

    # Return the badge with the icon inside it
    return format_html(
        '<i class="{}"></i>',
        icon_class
    )


admin_list._boolean_icon = custom_boolean_icon

register = template.Library()


@register.filter(name='replace')
def replace(value, args):
    key, val = args.split(',')
    return str(value).replace(key, val)


@register.simple_tag(name="popup_val")
def popup_val(request):
    return request.GET.get("_popup")


@register.filter
def sum_errors(formset):
    field_err = sum(len(form.errors) for form in formset)
    fieldset_err = len(formset.non_form_errors()) if hasattr(formset, 'non_form_errors') else 0
    return fieldset_err + field_err


def append_class_to_html_tag(html_tag, new_class):
    # Regex to find the class attribute
    pattern = r'(class="[^"]*)(")'

    # Check if 'class' attribute exists
    if re.search(pattern, html_tag):
        # If class attribute exists, append the new class
        html_tag = re.sub(pattern, r'\1 ' + new_class + r'\2', html_tag)
    else:
        # If no class attribute, add one before the closing '>'
        html_tag = re.sub(r'(<[^>]*)(>)', r'\1 class="' + new_class + r'"\2', html_tag)

    return mark_safe(html_tag)


@register.filter(name='add_class')
def add_class(value, css_class):
    if hasattr(value, 'as_widget'):
        return value.as_widget(attrs={'class': css_class})

    elif isinstance(value, str):
        return append_class_to_html_tag(value, css_class)

    return value


@register.filter(name='apply_class_to_widget')
def apply_class_to_widget(widget, new_classes):
    default_class = widget.get('attrs', {}).get('class', '')
    widget['attrs']['class'] = f"{default_class} {new_classes}"
    return widget


def get_value_by_key(query, key):
    # Remove the leading '?' if present
    if query.startswith('?'):
        query = query[1:]

    # Create a regex pattern to match the key and capture its value
    pattern = re.compile(rf"{re.escape(key)}=([^&]*)")

    # Search for the pattern in the query string
    match = pattern.search(query)

    if match:
        # If a match is found, decode the value and return it
        return unquote(match.group(1))

    return None  # Return None if the key is not found


@register.simple_tag
def is_active_choice(choice, spec):
    try:
        if spec.lookup_kwarg not in choice['query_string']:
            return ''
    except Exception:
        pass

    if choice.get('selected'):
        return 'selected'

    if hasattr(spec, "lookup_kwarg"):
        filter_key = spec.lookup_kwarg

    elif hasattr(spec, "parameter_name"):
        filter_key = spec.parameter_name

    else:
        filter_key = spec.field_generic

    if hasattr(spec, 'request'):
        filter_values = spec.request.GET.get(filter_key.replace('__exact', '__in'))
    else:
        filter_values = ""

    current_choice_value = get_value_by_key(choice['query_string'], filter_key)

    if current_choice_value and filter_values:
        if current_choice_value in filter_values.split(','):
            return 'selected'

    return ''


@register.simple_tag
def get_active_filters_count(cl):
    if hasattr(cl, 'filter_params'):
        return len(cl.filter_params)

    return 0


@register.simple_tag
def is_multiple_filter_choice(spec):
    not_multiple = []
    if hasattr(spec, 'field_generic'):
        if spec.field_generic.endswith('__'):
            return ''

    return 'multiple'


@register.simple_tag
def get_user_admin_change_url(user):
    try:
        User = get_user_model()
        app_label = User._meta.app_label
        model_name = User._meta.model_name
        return reverse(f"admin:{app_label}_{model_name}_change", args=[user.pk])
    except Exception:
        return '#'




import os
import subprocess

from django.conf import settings  # Import settings to get project root
from django.core.management.base import BaseCommand

__all__ = ["Command", ]


class Command(BaseCommand):
    help = 'Sets the project root directory in an environment variable and runs "npm run watch"'

    def handle(self, *args, **kwargs):
        # Get the project root directory from Django settings
        project_root = str(settings.BASE_DIR)

        # Get the module directory
        django_daisy_dir_path = os.path.abspath(__file__).replace(
            'django_daisy/management/commands/build_daisyui.py', ''
        )

        # Set the project root as an environment variable
        os.environ['DJANGO_ROOT_PROJECT_PATH'] = project_root

        os.chdir(django_daisy_dir_path)

        # Check if npm is installed
        if not self.is_npm_installed():
            self.stdout.write(self.style.WARNING("npm is not installed. Please install npm on your system."))

        # Check if npm packages are installed
        if not self.are_npm_packages_installed(django_daisy_dir_path):
            self.stdout.write(self.style.WARNING("npm packages are not installed. Installing packages..."))
            if not self.install_npm_packages():
                self.stderr.write(self.style.ERROR("Failed to install npm packages."))
                return
            self.stdout.write(self.style.SUCCESS("npm packages installed successfully."))

        try:
            # Run the npm command

            self.stdout.write(self.style.SUCCESS('Running "npm run watch"...'))

            subprocess.run(["npm", "run", "watch"], check=True)

        except subprocess.CalledProcessError as e:
            self.stderr.write(self.style.ERROR(f"Error running npm command: {e}"))
        except Exception as e:
            self.stderr.write(self.style.ERROR(f"An error occurred: {e}"))

    def is_npm_installed(self):
        """
        Check if npm is installed by running "npm --version" and catching any errors.
        Returns True if npm is installed, False otherwise.
        """
        try:
            subprocess.run(["npm", "--version"], check=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
            return True
        except (subprocess.CalledProcessError, FileNotFoundError):
            return False

    def are_npm_packages_installed(self, package_path):
        """
        Check if npm packages are installed by verifying the presence of node_modules directory.
        Returns True if packages are installed, False otherwise.
        """
        node_modules_path = os.path.join(package_path, '../node_modules')
        return os.path.exists(node_modules_path)

    def install_npm_packages(self):
        """
        Run "npm install" to install npm packages from package.json.
        Returns True if the packages were installed successfully, False otherwise.
        """
        try:
            subprocess.run(["npm", "install"], check=True)
            return True
        except subprocess.CalledProcessError as e:
            self.stderr.write(self.style.ERROR(f"Failed to install npm packages: {e}"))
            return False



from django.contrib import admin
from django.utils.translation import gettext_lazy as _


class LogentryAdmin(admin.ModelAdmin):
    list_display = (
        'content_type', '_action', 'action_time', 'user'
    )
    list_filter = (
        'content_type', 'user',
    )
    search_fields = ('user__username', )
    list_per_page = 50

    def has_change_permission(self, request, obj=None):
        return False

    def has_delete_permission(self, request, obj=None):
        return False

    def has_add_permission(self, request):
        return False

    def get_model_perms(self, request):
        return {}

    @admin.display(description=_('Action'))
    def _action(self, obj):
        return obj.get_action_flag_display()



from django import forms
from django.db import models


class CheckBoxModelField(models.CharField):
    def formfield(self, **kwargs):
        return forms.ChoiceField(
            choices=self.choices,
            widget=forms.RadioSelect,
            label=self.verbose_name,
        )



from django import forms
from django.db import models


class TinyEditorWidget(forms.Textarea):
    class Media:
        js = (
            'admin/fields/tiny_init.js',
        )


class TinyEditorField(models.TextField):

    def __init__(self, *args, column_type='mediumtext', inline=False, **kwargs):
        # specifies column type choices are text | mediumtext | longtext
        # default is mediumtext
        self.column_type = column_type
        self.inline = inline
        super().__init__(*args, **kwargs)

    def formfield(self, **kwargs):
        _class = "editor-field" if not self.inline else "editor-inline-field"
        kwargs['widget'] = TinyEditorWidget(attrs={'class': _class})
        return super().formfield(**{
            'max_length': self.max_length,
            **({} if self.choices is not None else {'widget': forms.Textarea}),
            **kwargs,
        })

    def db_type(self, connection):
        if connection.settings_dict['ENGINE'] == 'django.db.backends.mysql':
            return self.column_type
        elif connection.settings_dict['ENGINE'] == 'django.db.backends.postgresql':
            return 'text'

        return super().db_type(connection)
